package function.builtin.predicate;

import static sexpression.Nil.NIL;
import static sexpression.Symbol.T;

import function.*;
import sexpression.*;

@FunctionNames({ ">" })
public class NUMERIC_GREATER extends LispFunction {

    private ArgumentValidator argumentValidator;

    public NUMERIC_GREATER(String name) {
        this.argumentValidator = new ArgumentValidator(name);
        this.argumentValidator.setMinimumNumberOfArguments(1);
        this.argumentValidator.setEveryArgumentExpectedType(LispNumber.class);
    }

    @Override
    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        return callTailRecursive(argumentList);
    }

    private SExpression callTailRecursive(Cons argumentList) {
        Cons remainingArguments = (Cons) argumentList.getRest();

        if (remainingArguments.isNull())
            return T;

        SExpression firstArgument = argumentList.getFirst();
        SExpression secondArgument = remainingArguments.getFirst();
        LispNumber number1 = (LispNumber) firstArgument;
        LispNumber number2 = (LispNumber) secondArgument;

        if (!isFirstGreater(number1, number2))
            return NIL;

        return callTailRecursive(remainingArguments);
    }

    private boolean isFirstGreater(LispNumber number1, LispNumber number2) {
        return number1.getValue().compareTo(number2.getValue()) > 0;
    }

}
